package org.moire.ultrasonic.api.subsonic

import org.moire.ultrasonic.api.subsonic.response.GetAlbumResponse
import org.moire.ultrasonic.api.subsonic.response.GetArtistResponse
import org.moire.ultrasonic.api.subsonic.response.GetArtistsResponse
import org.moire.ultrasonic.api.subsonic.response.GetIndexesResponse
import org.moire.ultrasonic.api.subsonic.response.GetMusicDirectoryResponse
import org.moire.ultrasonic.api.subsonic.response.LicenseResponse
import org.moire.ultrasonic.api.subsonic.response.MusicFoldersResponse
import org.moire.ultrasonic.api.subsonic.response.SubsonicResponse
import retrofit2.Call
import retrofit2.http.GET
import retrofit2.http.Query

/**
 * Subsonic API calls.
 *
 * For methods description see [http://www.subsonic.org/pages/api.jsp].
 */
interface SubsonicAPIDefinition {
    @GET("ping.view")
    fun ping(): Call<SubsonicResponse>

    @GET("getLicense.view")
    fun getLicense(): Call<LicenseResponse>

    @GET("getMusicFolders.view")
    fun getMusicFolders(): Call<MusicFoldersResponse>

    @GET("getIndexes.view")
    fun getIndexes(@Query("musicFolderId") musicFolderId: Long?,
                   @Query("ifModifiedSince") ifModifiedSince: Long?): Call<GetIndexesResponse>

    @GET("getMusicDirectory.view")
    fun getMusicDirectory(@Query("id") id: Long): Call<GetMusicDirectoryResponse>

    @GET("getArtists.view")
    fun getArtists(@Query("musicFolderId") musicFolderId: Long?): Call<GetArtistsResponse>

    @GET("star.view")
    fun star(@Query("id") id: Long? = null,
             @Query("albumId") albumId: Long? = null,
             @Query("artistId") artistId: Long? = null): Call<SubsonicResponse>

    @GET("unstar.view")
    fun unstar(@Query("id") id: Long? = null,
               @Query("albumId") albumId: Long? = null,
               @Query("artistId") artistId: Long? = null): Call<SubsonicResponse>

    @GET("getArtist.view")
    fun getArtist(@Query("id") id: Long): Call<GetArtistResponse>

    @GET("getAlbum.view")
    fun getAlbum(@Query("id") id: Long): Call<GetAlbumResponse>
}
